#  ICE Revision: $Id: PlotRunner.py 7811 2007-08-23 12:18:56Z bgschaid $ 
"""
Class that implements pyFoamPlotRunner
"""

from PyFoamApplication import PyFoamApplication

from PyFoam.FoamInformation import changeFoamVersion

from PyFoam.Execution.GnuplotRunner import GnuplotRunner

from PyFoam.Execution.ParallelExecution import LAMMachine

from PyFoam.RunDictionary.SolutionDirectory import SolutionDirectory

from os import path

class PlotRunner(PyFoamApplication):
    def __init__(self):
        description="""
        runs an OpenFoam solver needs the usual 3 arguments (<solver>
        <directory> <case>) and passes them on (plus additional arguments).
        Output is sent to stdout and a logfile inside the case directory
        (PyFoamSolver.logfile) Information about the residuals is output as
        graphs
        
        If the directory contains a file customRegexp this is automatically
        read and the regular expressions in it are displayed
        """
        PyFoamApplication.__init__(self,description=description)

    def addOptions(self):
        self.parser.add_option("--frequency",
                               type="float",
                               dest="frequency",
                               default=1.,
                               help="The frequency with which output should be generated (in seconds)")
        
        self.parser.add_option("--persist",
                               action="store_true",
                               dest="persist",
                               default=True,
                               help="Gnuplot windows stay after interrupt")
        
        self.parser.add_option("--non-persist",
                               action="store_false",
                               dest="persist",
                               help="Gnuplot windows close after interrupt")
        
        self.parser.add_option("--raise",
                               action="store_true",
                               dest="raiseit",
                               help="Raise the Gnuplot windows after every replot")
        
        self.parser.add_option("--no-default",
                               action="store_true",
                               default=False,
                               dest="nodefault",
                               help="Switch off the default plots (linear, continuity and bound)")
        
        self.parser.add_option("--no-linear",
                               action="store_false",
                               default=True,
                               dest="linear",
                               help="Don't plot the linear solver convergence")
        
        self.parser.add_option("--no-continuity",
                               action="store_false",
                               default=True,
                               dest="cont",
                               help="Don't plot the continuity info")
        
        self.parser.add_option("--no-bound",
                               action="store_false",
                               default=True,
                               dest="bound",
                               help="Don't plot the bounding of variables")
        
        self.parser.add_option("--with-iterations",
                               action="store_true",
                               default=False,
                               dest="iterations",
                               help="Plot the number of iterations of the linear solver")
        
        self.parser.add_option("--with-courant",
                               action="store_true",
                               default=False,
                               dest="courant",
                               help="Plot the courant-numbers of the flow")
        
        self.parser.add_option("--with-execution",
                               action="store_true",
                               default=False,
                               dest="execution",
                               help="Plot the execution time of each time-step")
        
        self.parser.add_option("--with-deltat",
                               action="store_true",
                               default=False,
                               dest="deltaT",
                               help="'Plot the timestep-size time-step")
        
        self.parser.add_option("--with-all",
                               action="store_true",
                               default=False,
                               dest="withAll",
                               help="Switch all possible plots on")
        
        self.parser.add_option("--custom-regexp",
                               action="append",
                               default=None,
                               dest="customRegex",
                               help="Add a custom regular expression to be plotted (can be used more than once)")
        
        self.parser.add_option("--write-files",
                               action="store_true",
                               default=False,
                               dest="writeFiles",
                               help="Writes the parsed data to files")
        
        self.parser.add_option("--regexp-file",
                               action="store",
                               default=None,
                               dest="regexpFile",
                               help="A file with regulare expressions that are treated like the expressions given with --custom-regexp")
        
        self.parser.add_option("--procnr",
                               type="int",
                               dest="procnr",
                               default=None,
                               help="The number of processors the run should be started on")
        
        self.parser.add_option("--machinefile",
                               dest="machinefile",
                               default=None,
                               help="The machinefile that specifies the parallel machine")
        
        self.parser.add_option("--clear-case",
                               action="store_true",
                               default=False,
                               dest="clearCase",
                               help="Clear all timesteps except for the first before running")
        
        self.parser.add_option("--steady-run",
                               action="store_true",
                               default=False,
                               dest="steady",
                               help="This is a steady run. Stop it after convergence")
        
        self.parser.add_option("--progress",
                               action="store_true",
                               default=False,
                               dest="progress",
                               help="Only prints the progress of the simulation, but swallows all the other output")
        
        self.parser.add_option("--foamVersion",
                               dest="foamVersion",
                               default=None,
                               help="Change the OpenFOAM-version that is to be used")

    def addFileRegexps(self,fName):
        """Adds the lines from a file to the custom regular expressions
        @param fName: The name of the file"""
        f=open(fName)

        for l in f.readlines():
            l=l.strip()
            if l[0]=='"' and l[-1]=='"':
                l=l[1:-1]
            if len(l)>0:
                if self.opts.customRegex==None:
                    self.opts.customRegex=[]
                self.opts.customRegex.append(l)
        f.close()

        
    def run(self):
        if self.opts.foamVersion!=None:
            changeFoamVersion(self.opts.foamVersion)

        if self.opts.nodefault:
            self.opts.linear=False
            self.opts.cont=False
            self.opts.bound=False
            
        if self.opts.withAll:
            self.opts.linear=True
            self.opts.cont=True
            self.opts.bound=True
            self.opts.iterations=True
            self.opts.courant=True
            self.opts.execution=True
            self.opts.deltaT=True

        if self.opts.regexpFile!=None:
            self.addFileRegexps(self.opts.regexpFile)

        autoFile=path.join(self.parser.getArgs()[1],self.parser.getArgs()[2],"customRegexp")
        if path.exists(autoFile):
            print " Reading regular expressions from",autoFile
            self.addFileRegexps(autoFile)

        if self.opts.clearCase:
            print "Clearing out old timesteps ...."
            
            cName=self.parser.getArgs()[2]
            sol=SolutionDirectory(cName)
            sol.clearResults()

        lam=None
        if self.opts.procnr!=None or self.opts.machinefile!=None:
            lam=LAMMachine(machines=self.opts.machinefile,nr=self.opts.procnr)
        
        run=GnuplotRunner(argv=self.parser.getArgs(),smallestFreq=self.opts.frequency,persist=self.opts.persist,plotLinear=self.opts.linear,plotCont=self.opts.cont,plotBound=self.opts.bound,plotIterations=self.opts.iterations,plotCourant=self.opts.courant,plotExecution=self.opts.execution,plotDeltaT=self.opts.deltaT,customRegexp=self.opts.customRegex,writeFiles=self.opts.writeFiles,server=True,lam=lam,raiseit=self.opts.raiseit,steady=self.opts.steady,progress=self.opts.progress)
            
        run.start()
