#  ICE Revision: $Id: SolutionDirectory.py 7832 2007-08-28 13:07:26Z bgschaid $ 
"""Working with a solution directory"""

from PyFoam.Basics.Utilities import Utilities
from PyFoam.Basics.BasicFile import BasicFile

from os import listdir,path,mkdir
import tarfile,fnmatch

class SolutionDirectory(Utilities):
    """Represents a solution directory

    In the solution directory subdirectories whose names are numbers
    are assumed to be solutions for a specific time-step

    A sub-directory (called the Archive) is created to which solution
    data is copied"""
    
    def __init__(self,name,archive="ArchiveDir"):
        """@param name: Name of the solution directory
        @param archive: name of the directory where the lastToArchive-method
        should copy files, if None no archive is created"""

        self.name=name
        self.archive=None
        if archive!=None:
            self.archive=path.join(name,archive)
            if not path.exists(self.archive):
                mkdir(self.archive)

        self.backups=[]
        
        self.reread()

        self.essential=[self.systemDir(),self.constantDir(),self.initialDir()]

    def isValid(self):
        """Checks whether this is a valid case directory by looking for
        the system- and constant-directories and the controlDict-file"""
        if not path.exists(self.systemDir()):
            return False
        elif not path.isdir(self.systemDir()):
            return False
        elif not path.exists(self.constantDir()):
            return False
        elif not path.isdir(self.constantDir()):
            return False
        elif not path.exists(self.controlDict()):
            return False
        else:
            return True
        
    def addToClone(self,name):
        """add directory to the list that is needed to clone this case
        @param name: name of the subdirectory (the case directory is prepended)"""
        self.essential.append(path.join(self.name,name))

    def cloneCase(self,name,svnRemove=True):
        """create a clone of this case directory. Remove the target directory, if it already exists

        @param name: Name of the new case directory
        @param svnRemove: Look for .svn-directories and remove them
        @rtype: L{SolutionDirectory} or correct subclass
        @return: The target directory"""

        if path.exists(name):
            self.execute("rm -r "+name)
        mkdir(name)
        for d in self.essential:
            self.execute("cp -r "+d+" "+name)

        if svnRemove:
            self.execute("find "+name+" -name .svn -exec rm -rf {} \\; -prune")
            
        return self.__class__(name)

    def packCase(self,tarname,last=False,exclude=[],additional=[]):
        """Packs all the important files into a compressed tarfile.
        Uses the essential-list and excludes the .svn-directories.
        Also excludes files ending with ~
        @param tarname: the name of the tar-file
        @param last: add the last directory to the list of directories to be added
        @param exclude: List with additional glob filename-patterns to be excluded
        @param additional: List with additional glob filename-patterns
        that are to be added"""

        ex=["*~",".svn"]+exclude
        members=self.essential[:]
        if last:
            if self.getLast()!=self.first:
                members.append(self.latestDir())
        for p in additional:
            for f in listdir(self.name):
                if (f not in members) and fnmatch.fnmatch(f,p):
                    members.append(path.join(self.name,f))

        tar=tarfile.open(tarname,"w:gz")

        for m in members:
            self.addToTar(tar,m,exclude=ex)
            
        tar.close()

    def addToTar(self,tar,name,exclude=[]):
        """The workhorse for the packCase-method"""

        for e in exclude:
            if fnmatch.fnmatch(path.basename(name),e):
                return
            
        if path.isdir(name):
            for m in listdir(name):
                self.addToTar(tar,path.join(name,m),exclude=exclude)
        else:
            tar.add(name)
            
    def reread(self):
        """Rescan the directory for the time directories"""
        self.times=[]
        self.first=None
        self.last=None
        
        for f in listdir(self.name):
            try:
                val=float(f)
                self.times.append(f)
                if self.first==None:
                    self.first=f
                else:
                    if float(f)<float(self.first):
                        self.first=f
                if self.last==None:
                    self.last=f
                else:
                    if float(f)>float(self.last):
                        self.last=f
                        
            except ValueError:
                pass
            
        self.times.sort(self.sorttimes)
        
    def sorttimes(self,x,y):
        """Sort function for the solution files"""
        if(float(x)==float(y)):
            return 0
        elif float(x)<float(y):
            return -1
        else:
            return 1

    
    def getTimes(self):
        """ @return: List of all the available times"""
        self.reread()
        return self.times
        
    def addBackup(self,pth):
        """add file to list of files that are to be copied to the
        archive"""
        self.backups.append(path.join(self.name,pth))

    def getLast(self):
        """@return: the last time for which a solution exists
        @rtype: str"""
        self.reread()
        return self.last
    
    def lastToArchive(self,name):
        """copy the last solution (plus the backup-files to the
        archive)

        @param name: name of the sub-directory in the archive"""
        if self.archive==None:
            print "Warning: nor Archive-directory"
            return
        
        self.reread()
        fname=path.join(self.archive,name)
        if path.exists(fname):
            self.execute("rm -r "+fname)
        mkdir(fname)
        self.execute("cp -r "+path.join(self.name,self.last)+" "+fname)
        for f in self.backups:
            self.execute("cp -r "+f+" "+fname)
            
    def clearResults(self,after=None):
        """remove all time-directories after a certain time. If not time ist
        set the initial time is used
        @param after: time after which directories ar to be removed"""

        self.reread()
        
        if after==None:
            time=float(self.first)
        else:
            time=float(after)
            
        for f in self.times:
            if float(f)>time:
                self.execute("rm -r "+path.join(self.name,f))

    def clearPattern(self,glob):
        """Clear all files that fit a certain shell (glob) pattern
        @param glob: the pattern which the files are going to fit"""

        self.execute("rm -rf "+path.join(self.name,glob))
        
    def clearOther(self,processor=True,pyfoam=True):
        """Remove additional directories
        @param processor: remove the processorXX directories
        @param pyfoam: rremove all directories typically created by PyFoam"""

        if processor:
            self.clearPattern("processor?*")
        if pyfoam:
            self.clearPattern("PyFoam.?*")
            self.clearPattern("*?.analyzed")
            
    def clear(self,after=None,processor=True,pyfoam=True):
        """One-stop-shop to remove data
        @param after: time after which directories ar to be removed
        @param processor: remove the processorXX directories
        @param pyfoam: rremove all directories typically created by PyFoam"""
        self.clearResults(after=after)
        self.clearOther(processor=processor,pyfoam=pyfoam)
        
    def initialDir(self):
        """@return: the name of the first time-directory (==initial
        conditions
        @rtype: str"""
        if self.first:
            return path.join(self.name,self.first)
        else:
            return None
        
    def latestDir(self):
        """@param region: Specify the region for cases with more than 1 mesh
        @return: the name of the first last-directory (==simulation
        results)
        @rtype: str"""
        last=self.getLast()
        if last:
            return path.join(self.name,last)
        else:
            return None
        
    def constantDir(self,region=None):
        """@param region: Specify the region for cases with more than 1 mesh
        @return: the name of the C{constant}-directory
        @rtype: str"""
        if region:
            return path.join(self.name,"constant",region)
        else:
            return path.join(self.name,"constant")

    def systemDir(self,region=None):
        """@param region: Specify the region for cases with more than 1 mesh
        @return: the name of the C{system}-directory
        @rtype: str"""
        if region:
            return path.join(self.name,"system",region)
        else:
            return path.join(self.name,"system")

    def controlDict(self):
        """@param region: Specify the region for cases with more than 1 mesh
        @return: the name of the C{controlDict}
        @rtype: str"""
        return path.join(self.systemDir(),"controlDict")

    def polyMeshDir(self,region=None):
        """@param region: Specify the region for cases with more than 1 mesh
        @return: the name of the C{polyMesh}
        @rtype: str"""
        return path.join(self.constantDir(region=region),"polyMesh")

    def boundaryDict(self,region=None):
        """@param region: Specify the region for cases with more than 1 mesh
        @return: name of the C{boundary}-file
        @rtype: str"""
        return path.join(self.polyMeshDir(region=region),"boundary")
    
    def blockMesh(self,region=None):
        """@param region: Specify the region for cases with more than 1 mesh
        @return: the name of the C{blockMeshDict} if it exists. Returns
        an empty string if it doesn't
        @rtype: str"""
        p=path.join(self.polyMeshDir(region=region),"blockMeshDict")
        if path.exists(p):
            return p
        else:
            return ""
        
    def makeFile(self,name):
        """create a file in the solution directory and return a
        corresponding BasicFile-object

        @param name: Name of the file
        @rtype: L{BasicFile}"""
        return BasicFile(path.join(self.name,name))

    def getRegions(self):
        """Gets a list of all the available mesh regions by checking all
        directories in constant and using all those that have a polyMesh-subdirectory"""
        lst=[]
        for d in self.listDirectory(self.constantDir()):
            if path.isdir(path.join(self.constantDir(),d)):
                if path.exists(self.polyMeshDir(region=d)):
                    lst.append(d)
        lst.sort()
        return lst
    
class ChemkinSolutionDirectory(SolutionDirectory):
    """Solution directory with a directory for the Chemkin-files"""

    chemkinName = "chemkin"
    
    def __init__(self,name,archive="ArchiveDir"):
        SolutionDirectory.__init__(self,name,archive=archive)

        self.addToClone(self.chemkinName)

    def chemkinDir(self):
        """@rtype: str
        @return: The directory with the Chemkin-Files"""

        return path.join(self.name,self.chemkinName)
    
